"""IPv4 address asset."""
import dataclasses
from typing import Optional

from ostorlab.assets import asset


@dataclasses.dataclass
@asset.selector("v3.asset.ip.v4")
class IPv4(asset.Asset):
    """IPv4 Address target asset."""

    def __init__(
        self, host: str, version: Optional[int] = 4, mask: Optional[str] = None
    ):
        self.host = host
        self.version = version
        self.mask = mask

    def __str__(self) -> str:
        return f"{self.host}/{self.mask}"

    @property
    def proto_field(self) -> str:
        return "ipv4"



"""Proto to dict transformer.
The transformer maintains bytes format and doesn't go through intermediary JSON representation that is
both inefficient and resource intensive.

This code is credited to protobuf-to-dict. The app is written and maintained by Ben Hodgson, with significant
contributions from Nino Walker, Jonathan Klaassen, and Tristram Gräbener.
 """
from google.protobuf.descriptor import FieldDescriptor
from ostorlab import exceptions
from typing import Dict, Callable, Any


class UnrecognisedTypeError(exceptions.OstorlabError):
    """Field type is not recognised error."""


TYPE_CALLABLE_MAP = {
    FieldDescriptor.TYPE_DOUBLE: float,
    FieldDescriptor.TYPE_FLOAT: float,
    FieldDescriptor.TYPE_INT32: int,
    FieldDescriptor.TYPE_INT64: int,
    FieldDescriptor.TYPE_UINT32: int,
    FieldDescriptor.TYPE_UINT64: int,
    FieldDescriptor.TYPE_SINT32: int,
    FieldDescriptor.TYPE_SINT64: int,
    FieldDescriptor.TYPE_FIXED32: int,
    FieldDescriptor.TYPE_FIXED64: int,
    FieldDescriptor.TYPE_SFIXED32: int,
    FieldDescriptor.TYPE_SFIXED64: int,
    FieldDescriptor.TYPE_BOOL: bool,
    FieldDescriptor.TYPE_STRING: str,
    FieldDescriptor.TYPE_BYTES: bytes,
    FieldDescriptor.TYPE_ENUM: int,
}

EXTENSION_CONTAINER = "___X"


def _repeated(type_callable: Callable[[Any], Any]) -> Callable[[Any], Any]:
    """Function handler of repeated fields."""
    return lambda value_list: [type_callable(value) for value in value_list]


def _enum_label_name(field: Any, value: Any) -> str:
    """Function handler of enum fields. Generates name instead of int."""
    name: str = field.enum_type.values_by_number[int(value)].name
    return name


def _get_field_value_adaptor(
    pb: Any, field: Any, use_enum_labels: bool = False
) -> Callable[[Any], Any]:
    """Matches proto message or field to the proper handler."""
    if field.type == FieldDescriptor.TYPE_MESSAGE:
        # recursively encode protobuf sub-message
        return lambda pb: protobuf_to_dict(pb, use_enum_labels=use_enum_labels)

    if use_enum_labels and field.type == FieldDescriptor.TYPE_ENUM:
        return lambda value: _enum_label_name(field, value)

    if field.type in TYPE_CALLABLE_MAP:
        return TYPE_CALLABLE_MAP[field.type]

    raise UnrecognisedTypeError(
        f"Field {pb.__class__.__name__}.{field.name} has unrecognised type id {field.type}"
    )


def protobuf_to_dict(pb: Any, use_enum_labels: bool = False) -> Dict[Any, Any]:
    """Transforms Protobuf message to dict.

    The Method maintains bytes format and do not use intermediary representation like JSON.

    Args:
        pb: Protobuf message
        use_enum_labels: Use enum string label or return enum int.

    Returns:
        Dict representation of the protbuf.
    """
    result_dict = {}
    extensions = {}
    for field, value in pb.ListFields():
        type_callable = _get_field_value_adaptor(pb, field, use_enum_labels)
        if field.label == FieldDescriptor.LABEL_REPEATED:
            type_callable = _repeated(type_callable)

        if field.is_extension:
            extensions[str(field.number)] = type_callable(value)
            continue

        result_dict[field.name] = type_callable(value)

    if extensions:
        result_dict[EXTENSION_CONTAINER] = extensions
    return result_dict



"""Runtime registry that references all supported runtimes."""
from ostorlab import exceptions
from ostorlab.runtimes import runtime
from ostorlab.runtimes.local import runtime as local_runtime
from ostorlab.runtimes.lite_local import runtime as lite_local_runtime
from ostorlab.runtimes.cloud import runtime as cloud_runtime


class RuntimeNotFoundError(exceptions.OstorlabError):
    """Specified runtime is not found."""


def select_runtime(runtime_type: str, *args, **kwargs) -> runtime.Runtime:
    """Selects and instantiate runtime by type.

    Args:
        runtime_type: The type of runtime to create.
        *args: Runtime args.
        **kwargs: Runtime kwargs.

    Returns:
        Runtime instance.
    """
    if runtime_type == "local":
        return local_runtime.LocalRuntime(*args, **kwargs)
    elif runtime_type == "litelocal":
        return lite_local_runtime.LiteLocalRuntime(*args, **kwargs)
    elif runtime_type == "cloud":
        return cloud_runtime.CloudRuntime(*args, **kwargs)
    else:
        raise RuntimeNotFoundError(f"runtime type {runtime_type} not found")



"""Module responsible for establishing a connection with NATS."""

import asyncio
import datetime
import logging
import ssl
import sys
import traceback
from typing import Optional

from nats.js import errors as jetstream_errors
from nats import errors as nats_errors
import nats
from nats.js import api as js_api
from nats.js import client as js_client

from ostorlab.scanner.proto.scan._location import startAgentScan_pb2

DEFAULT_PENDING_BYTES_LIMIT = 400 * 1024 * 1024

DEFAULT_CONNECT_TIMEOUT = datetime.timedelta(seconds=20)

DEFAULT_MAX_INFLIGHT = 1

DEFAULT_ACK_WAIT = datetime.timedelta(seconds=180)


logger = logging.getLogger(__name__)


class ClientBusHandler:
    """Handler for establishing a connection with NATS and performing client operations."""

    def __init__(
        self,
        bus_url: str,
        cluster_id: str,
        name: str,
        tls_context: Optional[ssl.SSLContext] = None,
        loop=None,
    ):
        self._bus_url = bus_url
        self._cluster_id = cluster_id
        self._name = name
        self._nc: nats.NATS = nats.NATS()
        self._js: Optional[js_client.JetStreamContext] = None
        self._loop = loop or asyncio.get_event_loop()
        if tls_context is None and bus_url.startswith("tls://"):
            self._tls_context = ssl.create_default_context()
        else:
            self._tls_context = tls_context

    async def __aenter__(self) -> "ClientBusHandler":
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()

    async def connect(self, connect_timeout=DEFAULT_CONNECT_TIMEOUT.seconds):
        """
        Connect to the NATS server.

        Args:
            connect_timeout: Timeout for establishing the connection. Defaults to DEFAULT_CONNECT_TIMEOUT.
        """
        await self._nc.connect(
            self._bus_url,
            name=self._name,
            tls=self._tls_context,
            connect_timeout=connect_timeout,
            error_cb=self._error_cb,
            closed_cb=self._closed_cb,
            reconnected_cb=self._reconnected_cb,
        )
        self._js = self._nc.jetstream()

    async def add_stream(self, name, subjects):
        """
        Add a stream to the NATS server.

        Args:
            name: Name of the stream.
            subjects: Subjects for the stream.
        """
        await self._js.add_stream(
            name=name,
            subjects=subjects,
        )

    async def delete_stream(self, name):
        """
        Delete a stream from the NATS server.

        Args:
            name: Name of the stream.
        """
        try:
            await self._js.delete_stream(name=name)
        except jetstream_errors.ObjectDeletedError as e:
            logger.warning("error deleting stream %s: %s", name, e)

    async def close(self):
        """Close the connection to the NATS server."""
        await self._nc.close()

    async def _error_cb(self, e):
        logger.error("Error: %s", e)
        logger.error("Traceback: %s", traceback.print_exc())

    async def _closed_cb(self):
        logger.debug("Connection to Bus is closed.")

    async def _reconnected_cb(self):
        logger.debug("Reconnected to Bus at %s...", self._nc.connected_url.netloc)


class BusHandler(ClientBusHandler):
    """Handler for performing bus operations."""

    def __init__(
        self,
        bus_url: str,
        cluster_id: str,
        name: str,
        tls_context: Optional[ssl.SSLContext] = None,
        loop=None,
    ):
        super().__init__(
            bus_url=bus_url,
            cluster_id=cluster_id,
            name=name,
            tls_context=tls_context,
            loop=loop,
        )
        self._subjects_cb_map = {}
        self._last_message_received_time = datetime.datetime.now()
        self._pull_subscription = None

    async def ensure_running_handler(self):
        """Ensure Bus Handler is always running by checking the last received time of a message."""
        while True:
            try:
                logger.debug(
                    "checking last received message %s",
                    self._last_message_received_time,
                )
                await asyncio.sleep(60)
                if (
                    datetime.datetime.now()
                    > self._last_message_received_time + datetime.timedelta(hours=5)
                ):
                    logger.debug(
                        "too long after receiving the last message, restarting all subscriptions"
                    )
                    await self.close()
                    logger.debug("exiting process")
                    sys.exit(5)
            except jetstream_errors.ServiceUnavailableError as e:
                logger.error("Error in ensure running: %s", e)

    async def subscribe(
        self,
        subject: str,
        durable_name: Optional[str] = None,
        start_at: str = "first",
        max_inflight: int = DEFAULT_MAX_INFLIGHT,
        ack_wait: int = DEFAULT_ACK_WAIT.seconds,
    ):
        """Start bus subscription.

        subject: Subject for the Bus Streaming subscription.
        durable_name: Durable connection name.
        start_at: One of the following options:
           - 'new_only' (default)
           - 'first'
           - 'sequence'
           - 'last_received'
           - 'time'
        max_inflight: Max number of message in flight to client.
        ack_wait: How long to wait for an ack before being redelivered previous messages.
        """
        if start_at == "new_only":
            deliver_policy = js_api.DeliverPolicy.NEW
        elif start_at == "first":
            deliver_policy = js_api.DeliverPolicy.ALL
        elif start_at == "sequence":
            deliver_policy = js_api.DeliverPolicy.BY_START_SEQUENCE
        elif start_at == "last_received":
            deliver_policy = js_api.DeliverPolicy.LAST
        elif start_at == "time":
            deliver_policy = js_api.DeliverPolicy.BY_START_TIME
        else:
            deliver_policy = js_api.DeliverPolicy.NEW

        self._pull_subscription = await self._js.pull_subscribe(
            subject=subject,
            durable=durable_name,
            config=js_api.ConsumerConfig(
                durable_name=durable_name,
                ack_wait=ack_wait,
                deliver_policy=deliver_policy,
                max_ack_pending=max_inflight,
            ),
        )

    async def process_message(
        self,
    ):
        message = None
        request = None

        if self._pull_subscription is not None:
            try:
                logger.debug("Fetching messages.")
                msgs = await self._pull_subscription.fetch()
                for msg in msgs:
                    message = msg
                    logger.debug("Processing message: %s", message)
                    request = await self.parse_message(message)
                    yield message, request
            except nats_errors.TimeoutError:
                logger.debug("No message to fetch, sleeping..")
                await asyncio.sleep(1)
        yield message, request

    async def parse_message(self, message):
        logger.debug("process received message %s", message)
        self._last_message_received_time = datetime.datetime.now()
        request = startAgentScan_pb2.Message()
        request.ParseFromString(message.data)
        return request



"""Asset of type .AAB package file.
This module takes care of preparing a file of type .aab before injecting it to the runtime instance."""
import io
import logging
from typing import Tuple, Optional

import click

from ostorlab.assets import android_aab as android_aab_asset
from ostorlab.cli.scan.run import run
from ostorlab.cli import console as cli_console


console = cli_console.Console()
logger = logging.getLogger(__name__)


@run.run.command()
@click.option("--file", type=click.File(mode="rb"), multiple=True, required=False)
@click.option("--url", multiple=True, required=False)
@click.pass_context
def android_aab(
    ctx: click.core.Context,
    file: Optional[Tuple[io.FileIO]] = (),
    url: Optional[Tuple[str]] = (),
) -> None:
    """Run scan for android .AAB package file."""
    runtime = ctx.obj["runtime"]
    assets = []

    if url != () and file != ():
        console.error("Command accepts either path or source url of the aab file.")
        raise click.exceptions.Exit(2)
    if url == () and file == ():
        console.error("Command missing either file path or source url of the aab file.")
        raise click.exceptions.Exit(2)

    if file != ():
        for f in file:
            assets.append(
                android_aab_asset.AndroidAab(content=f.read(), path=str(f.name))
            )
    if url != ():
        for u in url:
            assets.append(android_aab_asset.AndroidAab(content_url=u))

    logger.debug("scanning assets %s", [str(asset) for asset in assets])
    runtime.scan(
        title=ctx.obj["title"],
        agent_group_definition=ctx.obj["agent_group_definition"],
        assets=assets,
    )



"""Module for the command run inside the group scan.
This module takes care of preparing the selected runtime and the lists of provided agents, before starting a scan.
Example of usage:
    - ostorlab scan run --agent=agent1 --agent=agent2 --title=test_scan [asset] [options]."""
import io
import logging
import requests
from typing import List

import click

from ostorlab.cli import install_agent
from ostorlab.cli.scan import scan
from ostorlab.runtimes import definitions
from ostorlab.runtimes import runtime
from ostorlab.cli import console as cli_console
from ostorlab.agent.schema import validator


console = cli_console.Console()

logger = logging.getLogger(__name__)


@scan.group(invoke_without_command=True)
@click.option(
    "--agent",
    multiple=True,
    help="List of agents keys. to use in the scan.",
    required=False,
)
@click.option("--title", "-t", help="Scan title.")
@click.option(
    "--agent-group-definition",
    "-g",
    type=click.File("r"),
    help="Path to agents group definition file (yaml).",
    required=False,
)
@click.option(
    "--install", "-i", help="Install missing agents.", is_flag=True, required=False
)
@click.option(
    "--follow",
    help="Follow logs of provided list of agents and services.",
    multiple=True,
    default=[],
)
@click.option(
    "--no-asset",
    help="Start the environment without injecting assets",
    is_flag=True,
    required=False,
)
@click.pass_context
def run(
    ctx: click.core.Context,
    agent: List[str],
    agent_group_definition: io.FileIO,
    title: str,
    install: bool,
    follow: List[str],
    no_asset: bool,
) -> None:
    """Start a new scan on a specific asset.\n
    Example:\n
        - ostorlab scan run --agent=agent/ostorlab/nmap --agent=agent/google/tsunami --title=test_scan ip 8.8.8.8
    """
    if no_asset is True and ctx.invoked_subcommand is not None:
        console.error(
            f"Sub-command {ctx.invoked_subcommand} specified with --no-asset flag."
        )
        raise click.exceptions.Exit(2)
    if no_asset is False and ctx.invoked_subcommand is None:
        console.error("Error: Missing command.")
        click.echo(ctx.get_help())
        raise click.exceptions.Exit(2)

    if agent:
        agents_settings: List[definitions.AgentSettings] = []
        for agent_key in agent:
            agents_settings.append(definitions.AgentSettings(key=agent_key))

        agent_group = definitions.AgentGroupDefinition(agents=agents_settings)
    elif agent_group_definition:
        try:
            agent_group = definitions.AgentGroupDefinition.from_yaml(
                agent_group_definition
            )
        except validator.ValidationError as e:
            console.error(f"{e}")
            raise click.ClickException("Invalid Agent Group Definition.") from e
    else:
        raise click.ClickException("Missing agent list or agent group definition.")

    runtime_instance: runtime.Runtime = ctx.obj["runtime"]
    # set list of log follow.
    runtime_instance.follow = follow

    if runtime_instance.can_run(agent_group_definition=agent_group):
        ctx.obj["agent_group_definition"] = agent_group
        ctx.obj["title"] = title
        if install is True:
            try:
                # Trigger both the runtime installation routine and install all the provided agents.
                runtime_instance.install()
                for ag in agent_group.agents:
                    try:
                        install_agent.install(ag.key, ag.version)
                    except install_agent.AgentDetailsNotFound:
                        console.warning(f"agent {ag.key} not found on the store")
            except requests.exceptions.ConnectionError as e:
                raise click.ClickException(f"Could not install the agents: {e}")

        if ctx.invoked_subcommand is None:
            runtime_instance.scan(
                title=ctx.obj["title"],
                agent_group_definition=ctx.obj["agent_group_definition"],
                assets=None,
            )
    else:
        raise click.ClickException(
            "The runtime does not support the provided agent list or group definition."
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/capture/request_response/request_response.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nOostorlab/agent/message/proto/v3/capture/request_response/request_response.proto\x12\x38ostorlab.agent.message.proto.v3.capture.request_response\"%\n\x06header\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x87\x01\n\x07request\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12Q\n\x07headers\x18\x03 \x03(\x0b\x32@.ostorlab.agent.message.proto.v3.capture.request_response.header\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\"\xd7\x01\n\x08response\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\x05\x12\x13\n\x0bstatus_text\x18\x03 \x01(\t\x12Q\n\x07headers\x18\x04 \x03(\x0b\x32@.ostorlab.agent.message.proto.v3.capture.request_response.header\x12\x11\n\tremote_ip\x18\x05 \x01(\t\x12\x13\n\x0bremote_port\x18\x06 \x01(\x05\x12\x10\n\x08protocol\x18\x07 \x01(\t\x12\x0c\n\x04\x62ody\x18\x08 \x01(\x0c\"\xb3\x01\n\x07Message\x12R\n\x07request\x18\x01 \x01(\x0b\x32\x41.ostorlab.agent.message.proto.v3.capture.request_response.request\x12T\n\x08response\x18\x02 \x01(\x0b\x32\x42.ostorlab.agent.message.proto.v3.capture.request_response.response')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.capture.request_response.request_response_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=141
  _HEADER._serialized_end=178
  _REQUEST._serialized_start=181
  _REQUEST._serialized_end=316
  _RESPONSE._serialized_start=319
  _RESPONSE._serialized_end=534
  _MESSAGE._serialized_start=537
  _MESSAGE._serialized_end=716
# @@protoc_insertion_point(module_scope)



# pylint: disable=C0103
"""Revision reflecting the state of databases created with ostorlab<0.11.2.
Useful as a starting point for future migrations.

Revision ID: 35cd577ef0e5
Revises:
Create Date: 2022-10-26 16:00:52.792734

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "35cd577ef0e5"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "scan",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("title", sa.String(length=255), nullable=True),
        sa.Column("asset", sa.String(length=255), nullable=True),
        sa.Column("created_time", sa.DateTime(), nullable=True),
        sa.Column(
            "progress",
            sa.Enum(
                "NOT_STARTED",
                "IN_PROGRESS",
                "STOPPED",
                "DONE",
                "ERROR",
                name="scanprogress",
            ),
            nullable=True,
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "scan_status",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("created_time", sa.DateTime(), nullable=True),
        sa.Column("key", sa.String(length=255), nullable=True),
        sa.Column("value", sa.Text(), nullable=True),
        sa.Column("scan_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["scan_id"],
            ["scan.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "vulnerability",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("technical_detail", sa.Text(), nullable=True),
        sa.Column(
            "risk_rating",
            sa.Enum(
                "HIGH",
                "MEDIUM",
                "LOW",
                "POTENTIALLY",
                "HARDENING",
                "SECURE",
                "IMPORTANT",
                "INFO",
                name="riskrating",
            ),
            nullable=True,
        ),
        sa.Column("cvss_v3_vector", sa.String(length=1024), nullable=True),
        sa.Column("dna", sa.String(length=256), nullable=True),
        sa.Column("title", sa.String(length=256), nullable=True),
        sa.Column("short_description", sa.Text(), nullable=True),
        sa.Column("description", sa.Text(), nullable=True),
        sa.Column("recommendation", sa.Text(), nullable=True),
        sa.Column("scan_id", sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(
            ["scan_id"],
            ["scan.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("vulnerability")
    op.drop_table("scan_status")
    op.drop_table("scan")
    # ### end Alembic commands ###



"""Common definition types."""
import dataclasses
from typing import Optional, Union


@dataclasses.dataclass
class Arg:
    """Data class holding a definition.

    The value is always bytes to support all arg values. The type is defined by the type attribute.
    """

    name: str
    type: str
    value: Optional[Union[bytes, int, float, str, bool]] = None
    description: Optional[str] = None


@dataclasses.dataclass
class PortMapping:
    """Data class defining a port mapping source to destination"""

    source_port: int
    destination_port: int


@dataclasses.dataclass
class ScannerState:
    """Current scanner state."""

    scanner_id: str
    scan_id: Optional[int]
    cpu_load: float
    memory_load: float
    total_cpu: int
    total_memory: int
    hostname: str
    ip: str



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/capture/filesystem/filesystem.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCostorlab/agent/message/proto/v3/capture/filesystem/filesystem.proto\x12\x32ostorlab.agent.message.proto.v3.capture.filesystem\"\xb6\x01\n\x07Message\x12H\n\x05\x65vent\x18\x01 \x01(\x0e\x32\x39.ostorlab.agent.message.proto.v3.capture.filesystem.Event\x12\x10\n\x08\x66ilename\x18\x02 \x01(\t\x12\x0b\n\x03pid\x18\x03 \x01(\x05\x12\x0b\n\x03uid\x18\x04 \x01(\x05\x12\x0b\n\x03gid\x18\x05 \x01(\x05\x12\x0c\n\x04mode\x18\x06 \x01(\x05\x12\x0c\n\x04ppid\x18\x07 \x01(\x05\x12\x0c\n\x04proc\x18\x08 \x01(\t*\x96\x02\n\x05\x45vent\x12\n\n\x06\x41\x43\x43\x45SS\x10\x01\x12\x08\n\x04OPEN\x10\x02\x12\r\n\tOPEN_EXEC\x10\x03\x12\n\n\x06\x41TTRIB\x10\x04\x12\n\n\x06\x43REATE\x10\x05\x12\n\n\x06\x44\x45LETE\x10\x06\x12\x0f\n\x0b\x44\x45LETE_SELF\x10\x07\x12\x0e\n\nMOVED_FROM\x10\x08\x12\x0c\n\x08MOVED_TO\x10\t\x12\r\n\tMOVE_SELF\x10\n\x12\x0f\n\x0b\x43LOSE_WRITE\x10\x0b\x12\x11\n\rCLOSE_NOWRITE\x10\x0c\x12\x0e\n\nQ_OVERFLOW\x10\r\x12\x0f\n\x0b\x41\x43\x43\x45SS_PERM\x10\x0e\x12\r\n\tOPEN_PERM\x10\x0f\x12\x12\n\x0eOPEN_EXEC_PERM\x10\x10\x12\t\n\x05\x43LOSE\x10\x11\x12\x08\n\x04MOVE\x10\x12\x12\t\n\x05ONDIR\x10\x13')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.capture.filesystem.filesystem_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _EVENT._serialized_start=309
  _EVENT._serialized_end=587
  _MESSAGE._serialized_start=124
  _MESSAGE._serialized_end=306
# @@protoc_insertion_point(module_scope)



"""Ostorlab main package."""
import logging

import click
from ostorlab.cli.rootcli import rootcli
from rich import console
from rich import logging as rich_logging

FORMAT = "%(message)s"
logging.basicConfig(
    level="ERROR",
    format=FORMAT,
    datefmt="[%X]",
    handlers=[
        rich_logging.RichHandler(
            rich_tracebacks=True,
            console=console.Console(width=255),
            tracebacks_suppress=[click],
        )
    ],
)

logger = logging.getLogger("CLI")


def main():
    rootcli(None)



"""models contain the database engine logic and all the db models and the related operations."""

import datetime
import enum
import json
import logging
import pathlib
from typing import Any, Dict, List, Optional
import types

import sqlalchemy
from sqlalchemy import orm
from sqlalchemy.ext import declarative
from sqlalchemy.engine.reflection import Inspector
from sqlalchemy.engine import base
from alembic import config
from alembic import script
from alembic import command as alembic_command
from alembic.runtime import migration
from alembic.util import exc as alembic_exceptions

from ostorlab import configuration_manager as config_manager
from ostorlab.cli import console as cli_console
from ostorlab.utils import risk_rating as utils_rik_rating


ENGINE_URL = f"sqlite:///{config_manager.ConfigurationManager().conf_path}/db.sqlite"
OSTORLAB_BASE_MIGRATION_ID = "35cd577ef0e5"


logger = logging.getLogger(__name__)
console = cli_console.Console()

metadata = sqlalchemy.MetaData()
Base = declarative.declarative_base(metadata=metadata)


class ScanProgress(enum.Enum):
    NOT_STARTED = "not_started"
    IN_PROGRESS = "in_progress"
    STOPPED = "stopped"
    DONE = "done"
    ERROR = "error"


class Database:
    """Handles all Database instantiation and calls."""

    def __init__(self):
        """Constructs the database engine."""
        self._db_engine = sqlalchemy.create_engine(ENGINE_URL)
        self._db_session = None
        self._alembic_ini_path = (
            pathlib.Path(__file__).parent.absolute() / "alembic.ini"
        )
        self._alembic_cfg = config.Config(str(self._alembic_ini_path))

    def __enter__(self) -> orm.Session:
        """Context manager enter method, resposible for migrating the local database and returning a session object."""
        self._migrate_local_db()
        return self._prepare_db_session()

    def __exit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_traceback: Optional[types.TracebackType],
    ) -> None:
        """Context manager exit method, responsible for closing the local database session"""
        if self._db_session is not None:
            self._db_session.close()

    def _is_db_populated(self, conn: base.Connection) -> bool:
        """Checks if the local database has tables."""
        inspector = Inspector.from_engine(conn)
        tables = inspector.get_table_names()
        return len(tables) != 0

    def _migrate_local_db(self) -> None:
        """Ensure the local database schema is up to date & run the migration otherwise."""
        try:
            alembic_script = script.ScriptDirectory.from_config(self._alembic_cfg)
            with self._db_engine.begin() as conn:
                context = migration.MigrationContext.configure(conn)
                # To ensure backward  compatibility with existing databases,
                # the next two lines do a fake migration of the base schema,
                # before applying the migrations from that point.
                if (
                    self._is_db_populated(conn)
                    and context.get_current_revision() is None
                ):
                    alembic_command.stamp(self._alembic_cfg, OSTORLAB_BASE_MIGRATION_ID)

                if context.get_current_revision() != alembic_script.get_current_head():
                    alembic_command.upgrade(self._alembic_cfg, "head")
        except (alembic_exceptions.CommandError, ValueError) as e:
            console.error(f"Error while migrating the local database: {str(e)}")

    def _prepare_db_session(self) -> orm.Session:
        """Returns a Session singleton to run queries on the db engine."""
        if self._db_session is None:
            session_maker = orm.sessionmaker(expire_on_commit=False)
            session_maker.configure(bind=self._db_engine)
            self._db_session = session_maker()
            return self._db_session
        else:
            return self._db_session

    def create_db_tables(self):
        """Create the database tables."""
        with self._db_engine.begin() as conn:
            metadata.create_all(conn)
            logger.info("Tables created")

    def drop_db_tables(self):
        """Drop the database tables."""
        metadata.drop_all(self._db_engine)
        logger.info("Tables dropped")


class Scan(Base):
    """The Scan model"""

    __tablename__ = "scan"
    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
    title = sqlalchemy.Column(sqlalchemy.String(255))
    asset = sqlalchemy.Column(sqlalchemy.String(255))
    created_time = sqlalchemy.Column(sqlalchemy.DateTime)
    progress = sqlalchemy.Column(sqlalchemy.Enum(ScanProgress))

    @staticmethod
    def create(asset, title: str = ""):
        """Persist the scan in the database.

        Args:
            title: Scan title.
        Returns:
            Scan object.
        """
        with Database() as session:
            scan = Scan(
                title=title,
                asset=asset,
                created_time=datetime.datetime.now(),
                progress="NOT_STARTED",
            )
            session.add(scan)
            session.commit()
            return scan


class Vulnerability(Base):
    """The Vulnerability model"""

    __tablename__ = "vulnerability"
    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
    technical_detail = sqlalchemy.Column(sqlalchemy.Text)
    risk_rating = sqlalchemy.Column(sqlalchemy.Enum(utils_rik_rating.RiskRating))
    cvss_v3_vector = sqlalchemy.Column(sqlalchemy.String(1024))
    dna = sqlalchemy.Column(sqlalchemy.String(256))
    title = sqlalchemy.Column(sqlalchemy.String(256))
    short_description = sqlalchemy.Column(sqlalchemy.Text)
    description = sqlalchemy.Column(sqlalchemy.Text)
    recommendation = sqlalchemy.Column(sqlalchemy.Text)
    references = sqlalchemy.Column(sqlalchemy.Text)
    scan_id = sqlalchemy.Column(sqlalchemy.Integer, sqlalchemy.ForeignKey("scan.id"))
    location = sqlalchemy.Column(sqlalchemy.Text)

    @staticmethod
    def _prepare_references_markdown(references: List[Dict[str, str]]) -> str:
        """Returns a markdown display of the references of a vulnerability."""
        if references is None or len(references) == 0:
            return ""
        references_markdwon_value = ""

        for reference in references:
            title = reference.get("title", "")
            url = reference.get("url", "")
            reference_markdwon = f"{title}: {url}  \n"
            references_markdwon_value += reference_markdwon

        return references_markdwon_value

    @staticmethod
    def _prepare_vuln_location_markdown(location: Dict[str, Any]) -> str:
        """Returns a markdown display of the exact target where the vulnerability was found."""
        if location is None:
            return ""
        location_markdwon_value = ""
        if location.get("domain_name") is not None:
            domain_name = location["domain_name"].get("name")
            location_markdwon_value = f"Domain: `{domain_name}`\n"
        elif location.get("ipv4") is not None:
            host = location["ipv4"].get("host")
            location_markdwon_value = f"IPv4: `{host}`\n"
        elif location.get("ipv6") is not None:
            host = location["ipv6"].get("host")
            location_markdwon_value = f"IPv6: `{host}`\n"
        elif location.get("android_store") is not None:
            package_name = location["android_store"].get("package_name")
            location_markdwon_value = f"Android: `{package_name}`\n"
        elif location.get("ios_store") is not None:
            bundle_id = location["ios_store"].get("bundle_id")
            location_markdwon_value = f"iOS: `{bundle_id}`\n"
        else:
            location_markdwon_value = f"Asset: `{json.dumps(location, indent=4)}`\n"

        for metadata_dict in location.get("metadata", []):
            metad_type = metadata_dict.get("type")
            metad_value = metadata_dict.get("value")
            location_markdwon_value += f"{metad_type}: {metad_value}  \n"
        return location_markdwon_value

    @staticmethod
    def create(
        scan_id: int,
        title: str,
        short_description: str,
        description: str,
        recommendation: str,
        technical_detail: str,
        risk_rating: str,
        cvss_v3_vector: str,
        dna: str,
        references: Optional[List[Dict[str, str]]] = None,
        location: Optional[Dict[str, Any]] = None,
    ):
        """Persist the vulnerability in the database.

        Args:
            scan_id: Scan id of the vulnerability.
            title: Vulnerability title.
            false_positive: Boolean to True if the vulnerability is a false positive.
            dna: A unique string to determine a vulnerability and avoid duplicates in the database.
            cvss_v3_vector: Common vulnerability scoring system value.
            risk_rating: Vulnerability risk rating.
            technical_detail: Technical details to demonstrate the finding.
            description: A generic description of the vulnerability.
            short_description: A short description of the vulnerability.
            recommendation: How to address or avoid the vulnerability
            references: List of references for extra information about the vulnerability.
            location: In which exact target the vulnerability was found.
        Returns:
            Vulnerability object.
        """
        references = Vulnerability._prepare_references_markdown(references)
        vuln_location = Vulnerability._prepare_vuln_location_markdown(location)
        vuln = Vulnerability(
            scan_id=scan_id,
            title=title,
            short_description=short_description,
            description=description,
            recommendation=recommendation,
            references=references,
            technical_detail=technical_detail,
            risk_rating=risk_rating,
            cvss_v3_vector=cvss_v3_vector,
            dna=dna,
            location=vuln_location,
        )

        with Database() as session:
            session.add(vuln)
            session.commit()

        return vuln


class ScanStatus(Base):
    """The Scan model"""

    __tablename__ = "scan_status"
    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
    created_time = sqlalchemy.Column(sqlalchemy.DateTime)
    key = sqlalchemy.Column(sqlalchemy.String(255))
    value = sqlalchemy.Column(sqlalchemy.Text)
    scan_id = sqlalchemy.Column(sqlalchemy.Integer, sqlalchemy.ForeignKey("scan.id"))

    @staticmethod
    def create(key: str, value: str, scan_id: int):
        """Persist the scan status in the database.

        Args:
            key: Scan status key.
            value: Scan status value.
        Returns:
            Scan status object.
        """
        scan_status = ScanStatus(
            key=key, created_time=datetime.datetime.now(), value=value, scan_id=scan_id
        )
        with Database() as session:
            session.add(scan_status)
            session.commit()
            return scan_status



"""Ios bundle_id target asset"""
import dataclasses

from ostorlab.assets import asset


@dataclasses.dataclass
@asset.selector("v3.asset.store.ios_store")
class IOSStore(asset.Asset):
    """Ios bundle target asset."""

    def __init__(self, bundle_id: str):
        self.bundle_id = bundle_id

    def __str__(self) -> str:
        return f"iOS Store ({self.bundle_id})"

    @property
    def proto_field(self) -> str:
        return "ios_store"



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ostorlab/agent/message/proto/v3/asset/domain_name/whois/whois.proto
# Protobuf Python Version: 4.25.0-rc2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCostorlab/agent/message/proto/v3/asset/domain_name/whois/whois.proto\x12\x37ostorlab.agent.message.proto.v3.asset.domain_name.whois\"\xf0\x02\n\x07Message\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tregistrar\x18\x02 \x01(\t\x12\x14\n\x0cwhois_server\x18\x03 \x01(\t\x12\x14\n\x0creferral_url\x18\x04 \x01(\t\x12\x14\n\x0cupdated_date\x18\x05 \x03(\t\x12\x15\n\rcreation_date\x18\x06 \x03(\t\x12\x17\n\x0f\x65xpiration_date\x18\x07 \x03(\t\x12\x14\n\x0cname_servers\x18\x08 \x03(\t\x12\x0e\n\x06status\x18\t \x03(\t\x12\x0e\n\x06\x65mails\x18\n \x03(\t\x12\x0e\n\x06\x64nssec\x18\x0b \x03(\t\x12\x18\n\x0c\x63ontact_name\x18\x0c \x01(\tB\x02\x18\x01\x12\x0b\n\x03org\x18\r \x01(\t\x12\x0f\n\x07\x61\x64\x64ress\x18\x0e \x01(\t\x12\x0c\n\x04\x63ity\x18\x0f \x01(\t\x12\r\n\x05state\x18\x10 \x01(\t\x12\x0f\n\x07zipcode\x18\x11 \x01(\t\x12\x0f\n\x07\x63ountry\x18\x12 \x01(\t\x12\x15\n\rcontact_names\x18\x13 \x03(\t')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ostorlab.agent.message.proto.v3.asset.domain_name.whois.whois_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_MESSAGE'].fields_by_name['contact_name']._options = None
  _globals['_MESSAGE'].fields_by_name['contact_name']._serialized_options = b'\030\001'
  _globals['_MESSAGE']._serialized_start=129
  _globals['_MESSAGE']._serialized_end=497
# @@protoc_insertion_point(module_scope)
